#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#-------------------------------------------------------------------------------
# This file is part of Mentat system (https://mentat.cesnet.cz/).
#
# Copyright (C) since 2011 CESNET, z.s.p.o (http://www.ces.net/)
# Use of this source is governed by the MIT license, see LICENSE file.
#-------------------------------------------------------------------------------


"""
Utility script for ordering reports generated by mentat-reporter.py module into
subdirectories. This will result in lower total number of files in report directory
(there might me hundreds of thousands of files and things could get pretty slow).
"""


__author__  = "Jan Mach <jan.mach@cesnet.cz>"


import os
import logging
import datetime

import pyzenkit.jsonconf
import mentat.const

#-------------------------------------------------------------------------------

def setup_logging():
    """
    Crude setup of both file and console logging.
    """
    logger = logging.getLogger('mentat-reports-order.py')
    logger.setLevel(logging.DEBUG)
    fhnd = logging.FileHandler('/var/log/mentat-reports-order.py.log')
    fhnd.setLevel(logging.DEBUG)
    chnd = logging.StreamHandler()
    chnd.setLevel(logging.INFO)
    formatter = logging.Formatter('%(asctime)s %(name)s %(levelname)s: %(message)s')
    fhnd.setFormatter(formatter)
    chnd.setFormatter(formatter)
    logger.addHandler(fhnd)
    logger.addHandler(chnd)
    return logger

#-------------------------------------------------------------------------------

#
# Setup logging mechanism.
#
LOGGER = setup_logging()

DT_START = datetime.datetime.now()
LOGGER.info("Report reordering started")

#
# Load reporting configurations.
#
CONFIG = pyzenkit.jsonconf.json_load('/etc/mentat/core/reporting.json.conf')
REPORTS_DIR = CONFIG['__core__reporter']['reports_dir']
LOGGER.info("Ordering reports in %s", REPORTS_DIR)

for filename in os.listdir(REPORTS_DIR):
    targetdirpath = mentat.const.construct_report_dirpath(REPORTS_DIR, filename)

    sourcepath = os.path.join(REPORTS_DIR, filename)
    targetpath = os.path.join(targetdirpath, filename)
    if not os.path.isfile(sourcepath):
        continue

    LOGGER.debug("Moving report from %s to %s", sourcepath, targetpath)
    while True:
        try:
            os.rename(sourcepath, targetpath)
            LOGGER.info("Moved report from %s to %s", sourcepath, targetpath)
            break

        except FileNotFoundError:
            os.mkdir(targetdirpath)
            LOGGER.info("Created subdirectory %s", targetdirpath)
